/**
 * JASMINe
 * Copyright (C) 2011 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: CorrelateCollector.java 8848 2011-09-13 08:10:24Z danesa $
 * --------------------------------------------------------------------------
 */

package org.ow2.jasmine.probe.collectors.correlate.internal;

import org.ow2.jasmine.probe.JasmineIndicatorValue;
import org.ow2.jasmine.probe.JasmineIndicator;
import org.ow2.jasmine.probe.JasmineSingleNumberResult;
import org.ow2.jasmine.probe.JasmineSingleResult;
import org.ow2.jasmine.probe.collector.JasmineCollector;
import org.ow2.jasmine.probe.collector.JasmineCollectorException;
import org.ow2.jasmine.probe.collectors.JCollector;

import java.util.ArrayList;
import java.util.Collection;

/**
 * Correlate Collector Implementation
 * @author durieuxp
 */
public class CorrelateCollector extends JCollector {

    private int ope;

    private Collection<JasmineCollector> sources;

    /**
     * Constructor
     * @param key
     * @param indic
     * @param period
     * @param ope
     * @param sources
     */
    public CorrelateCollector(String key, JasmineIndicator indic, int period, int ope, Collection<JasmineCollector> sources) {
        super(key, indic, period);
        this.ope = ope;
        this.sources = sources;
    }

    /**
     * return the last result for this indicator
     * @return JasmineIndicatorValue
     * @throws JasmineCollectorException
     */
    @Override
    public JasmineIndicatorValue getLastResult() throws JasmineCollectorException {
        logger.debug("");
        JasmineIndicatorValue jiv = new JasmineIndicatorValue();
        JasmineSingleResult jsr = new JasmineSingleNumberResult();
        long timestamp = System.currentTimeMillis();
        Number result = 0;
        Collection<Number> liste = new ArrayList<Number>();
        Number number1 = null;
        Number number2 = null;
        int count = 0;
        for (JasmineCollector elem : sources) {
            JasmineIndicatorValue jivs = elem.getLastResult();
            if (jivs == null) {
                logger.warn("No result available on source indicator");
                return null;
            }
            if (jivs.getValues().size() > 1) {
                logger.warn("Correlation on multivalued indicator not supported");
                throw new JasmineCollectorException("Correlation on multivalued indicator not supported");
            }
            for (JasmineSingleResult jsr1 : jivs.getValues()) {
                // only 1 value per JasmineIndicatorValue
                liste.add((Number) jsr1.getValue());
                if (count == 0) {
                    number1 = (Number) jsr1.getValue();
                } else if (count == 1) {
                    number2 = (Number) jsr1.getValue();
                }
                break;
            }
            count++;
        }
        switch (ope) {
            case CorrelateCollectorService.OP_ADD:
                result = addValues(liste);
                break;
            case CorrelateCollectorService.OP_SUB:
                result = diffValues(number1, number2);
                break;
            case CorrelateCollectorService.OP_MUL:
                result = multiplyValues(liste);
                break;
            case CorrelateCollectorService.OP_DIV:
                result = divideValues(number1, number2);
                break;
            case CorrelateCollectorService.OP_PERCENT:
                result = 100 * divideValues(number1, number2);
                break;
            default:
                throw new JasmineCollectorException("Bad correlate operation");
        }
        if (indicator.getScale() != 1) {
            result =  divideValues(result, indicator.getScale());
        }
        jsr.setTimestamp(timestamp);
        jsr.setValue(result);
        jsr.setName(indicator.getName());
        jiv.setName(indicator.getName());
        jiv.addValue(jsr);
        return jiv;
    }

    /**
     * Stop polling
     */
    public void stopPolling() {
        logger.debug("");
        // Nothing to do
    }

    /**
     * Start polling
     */
    @Override
    public void startPolling() {
        logger.debug("");
        // nothing to do
    }

    // ------------------------------------------------------------------------------------
    // private methods
    // ------------------------------------------------------------------------------------

}
